@@ -4,6 +4,7 @@ module Compare_core = Patdiff.Compare_core
 module Configuration = Patdiff.Configuration
 module Format = Patdiff.Format
 module Output = Patdiff.Output
+module Pcre = Re.Pcre
 
 let summary =
   {|Compare two files (or process a diff read in on stdin) using the
@@ -38,10 +39,11 @@ module Args = struct
     ; next_file : string
     ; prev_alt_opt : string option option
     ; next_alt_opt : string option option
-    ; include_ : string list
-    ; exclude : string list
+    ; include_ : Re.re list
+    ; exclude : Re.re list
     ; location_style : Format.Location_style.t option
     ; warn_if_no_trailing_newline_in_both : bool option
+    ; find_moves_opt : bool option
     }
 
   type t =
@@ -98,6 +100,7 @@ let override config (args : Args.compare_flags) =
     ?produce_unified_lines:args.produce_unified_lines_opt
     ?float_tolerance:args.float_tolerance_opt
     ?keep_ws:args.keep_ws_opt
+    ?find_moves:args.find_moves_opt
     ?split_long_lines:args.split_long_lines_opt
     ?interleave:args.interleave_opt
     ?assume_text:args.assume_text_opt
@@ -147,12 +150,12 @@ let compare_main (args : Args.compare_flags) =
       | { include_; exclude; _ } ->
         Some
           (fun (s, stat) ->
-             match stat.Unix.st_kind with
-             | Unix.S_REG ->
-               List.for_all exclude ~f:(fun pat -> not (Pcre.pmatch s ~pat))
-               && (List.is_empty include_
-                   || List.exists include_ ~f:(fun pat -> Pcre.pmatch s ~pat))
-             | _ -> true)
+            match stat.Unix.st_kind with
+            | Unix.S_REG ->
+              List.for_all exclude ~f:(fun rex -> not (Pcre.pmatch s ~rex))
+              && (List.is_empty include_
+                  || List.exists include_ ~f:(fun rex -> Pcre.pmatch s ~rex))
+            | _ -> true)
     in
     Compare_core.diff_dirs ~prev_dir:prev_file ~next_dir:next_file config ~file_filter
   | false, false ->
@@ -325,12 +328,12 @@ let command =
      and include_ =
        flag
          "include"
-         (listed string)
+         (listed (Arg_type.map string ~f:Pcre.regexp))
          ~doc:"REGEXP include files matching this pattern when comparing two directories"
      and exclude =
        flag
          "exclude"
-         (listed string)
+         (listed (Arg_type.map string ~f:Pcre.regexp))
          ~doc:
            "REGEXP exclude files matching this pattern when comparing two directories \
             (overrides include patterns)"
@@ -353,6 +356,12 @@ let command =
               "BOOL warn when neither file ends in a newline, even though this does not \
                constitute a diff (default: read from config, or %b)"
               Configuration.warn_if_no_trailing_newline_in_both_default)
+     and find_moves_opt =
+       flag_no_arg
+         "find-moves"
+         ~doc:
+           "Try to find and render moves. If -dont-produce-unified-lines is set moves \
+            will not be found."
      and files =
        anon
          (maybe
@@ -390,10 +399,13 @@ let command =
              ; exclude
              ; location_style
              ; warn_if_no_trailing_newline_in_both
+             ; find_moves_opt =
+                 (match produce_unified_lines_opt with
+                  | None | Some true -> find_moves_opt
+                  | Some false -> Some false)
              ; prev_file
              ; next_file
              }
        in
        main args)
 ;;
-
