@@ -28,6 +28,7 @@ type t =
   ; produce_unified_lines : bool
   ; unrefined : bool
   ; keep_ws : bool
+  ; find_moves : bool
   ; split_long_lines : bool
   ; interleave : bool
   ; assume_text : bool
@@ -42,9 +43,9 @@ type t =
   ; next_alt : string option
   ; location_style : Format.Location_style.t
   ; warn_if_no_trailing_newline_in_both : bool
-                                          [@default warn_if_no_trailing_newline_in_both_default] [@sexp_drop_default.equal]
+       [@default warn_if_no_trailing_newline_in_both_default] [@sexp_drop_default.equal]
   }
-[@@deriving compare, fields, sexp_of]
+[@@deriving compare, fields ~iterators:(iter, map), sexp_of]
 
 let invariant t =
   Invariant.invariant [%here] t [%sexp_of: t] (fun () ->
@@ -59,6 +60,7 @@ let invariant t =
       ~produce_unified_lines:ignore
       ~unrefined:ignore
       ~keep_ws:ignore
+      ~find_moves:ignore
       ~interleave:ignore
       ~assume_text:ignore
       ~split_long_lines:ignore
@@ -86,26 +88,27 @@ let invariant t =
 ;;
 
 let create_exn
-      ~output
-      ~rules
-      ~float_tolerance
-      ~produce_unified_lines
-      ~unrefined
-      ~keep_ws
-      ~split_long_lines
-      ~interleave
-      ~assume_text
-      ~context
-      ~line_big_enough
-      ~word_big_enough
-      ~shallow
-      ~quiet
-      ~double_check
-      ~mask_uniques
-      ~prev_alt
-      ~next_alt
-      ~location_style
-      ~warn_if_no_trailing_newline_in_both
+  ~output
+  ~rules
+  ~float_tolerance
+  ~produce_unified_lines
+  ~unrefined
+  ~keep_ws
+  ~find_moves
+  ~split_long_lines
+  ~interleave
+  ~assume_text
+  ~context
+  ~line_big_enough
+  ~word_big_enough
+  ~shallow
+  ~quiet
+  ~double_check
+  ~mask_uniques
+  ~prev_alt
+  ~next_alt
+  ~location_style
+  ~warn_if_no_trailing_newline_in_both
   =
   let t =
     { output
@@ -114,6 +117,7 @@ let create_exn
     ; produce_unified_lines
     ; unrefined
     ; keep_ws
+    ; find_moves
     ; split_long_lines
     ; interleave
     ; assume_text
@@ -135,27 +139,28 @@ let create_exn
 ;;
 
 let override
-      ?output
-      ?rules
-      ?float_tolerance
-      ?produce_unified_lines
-      ?unrefined
-      ?keep_ws
-      ?split_long_lines
-      ?interleave
-      ?assume_text
-      ?context
-      ?line_big_enough
-      ?word_big_enough
-      ?shallow
-      ?quiet
-      ?double_check
-      ?mask_uniques
-      ?prev_alt
-      ?next_alt
-      ?location_style
-      ?warn_if_no_trailing_newline_in_both
-      t
+  ?output
+  ?rules
+  ?float_tolerance
+  ?produce_unified_lines
+  ?unrefined
+  ?keep_ws
+  ?find_moves
+  ?split_long_lines
+  ?interleave
+  ?assume_text
+  ?context
+  ?line_big_enough
+  ?word_big_enough
+  ?shallow
+  ?quiet
+  ?double_check
+  ?mask_uniques
+  ?prev_alt
+  ?next_alt
+  ?location_style
+  ?warn_if_no_trailing_newline_in_both
+  t
   =
   let output = Option.value ~default:t.output output in
   let unrefined =
@@ -170,6 +175,7 @@ let override
       ~produce_unified_lines:(value produce_unified_lines)
       ~unrefined:(const unrefined)
       ~keep_ws:(value keep_ws)
+      ~find_moves:(value find_moves)
       ~interleave:(value interleave)
       ~assume_text:(value assume_text)
       ~split_long_lines:(value split_long_lines)
@@ -211,6 +217,7 @@ let default =
       ; word_same_prev = Format.Rule.create [ Dim ]
       ; word_same_next = Format.Rule.blank
       ; word_same_unified = Format.Rule.blank
+      ; word_same_unified_in_move = Format.Rule.create [ Fg Cyan ]
       ; word_prev = Format.Rule.create [ Fg Red ]
       ; word_next = Format.Rule.create [ Fg Green ]
       ; hunk =
@@ -228,11 +235,32 @@ let default =
           Format.Rule.create
             [ Bold ]
             ~pre:(Format.Rule.Affix.create "++++++ " ~styles:[ Fg Green ])
+      ; moved_from_prev =
+          Format.Rule.create
+            [ Fg Magenta ]
+            ~pre:(Format.Rule.Affix.create "<|" ~styles:[ Bg Magenta; Fg Black ])
+      ; moved_to_next =
+          Format.Rule.create
+            [ Fg Cyan ]
+            ~pre:(Format.Rule.Affix.create ">|" ~styles:[ Bg Cyan; Fg Black ])
+      ; removed_in_move =
+          Format.Rule.create
+            [ Fg Red ]
+            ~pre:(Format.Rule.Affix.create ">|" ~styles:[ Bg Red; Fg Black ])
+      ; added_in_move =
+          Format.Rule.create
+            [ Fg Green ]
+            ~pre:(Format.Rule.Affix.create ">|" ~styles:[ Bg Green; Fg Black ])
+      ; line_unified_in_move =
+          Format.Rule.create
+            []
+            ~pre:(Format.Rule.Affix.create ">|" ~styles:[ Bg Yellow; Fg Black ])
       }
   ; float_tolerance = None
   ; produce_unified_lines = true
   ; unrefined = false
   ; keep_ws = false
+  ; find_moves = false
   ; split_long_lines = false
   ; interleave = true
   ; assume_text = false
